/****************************************************************************
 *   Copyright (C) 2015-2016 by Savoir-faire Linux                               *
 *   Author : Emmanuel Lepage Vallee <emmanuel.lepage@savoirfairelinux.com> *
 *                                                                          *
 *   This library is free software; you can redistribute it and/or          *
 *   modify it under the terms of the GNU Lesser General Public             *
 *   License as published by the Free Software Foundation; either           *
 *   version 2.1 of the License, or (at your option) any later version.     *
 *                                                                          *
 *   This library is distributed in the hope that it will be useful,        *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
 *   Lesser General Public License for more details.                        *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#pragma once

#include <typedefs.h>

class QAbstractItemModel;
class QItemSelectionModel;

namespace Video {

/**
 * This class is used to simplify the configuration process.
 * Currently, every devices have their own model tree. This
 * proxy flatten the three to the clients don't have to
 * implement the managing logic.
 */
class LIB_EXPORT ConfigurationProxy {
public:
   static QAbstractItemModel& deviceModel    ();
   static QAbstractItemModel& channelModel   ();
   static QAbstractItemModel& resolutionModel();
   static QAbstractItemModel& rateModel      ();

   static QItemSelectionModel& deviceSelectionModel    ();
   static QItemSelectionModel& channelSelectionModel   ();
   static QItemSelectionModel& resolutionSelectionModel();
   static QItemSelectionModel& rateSelectionModel      ();

   static bool isDecodingAccelerated();
   static void setDecodingAccelerated(bool state);
};

} //namespace Video
